----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 09.02.2017 16:43:07
--
-- Module Name: TMDS decoder
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      Module decode TMDS signal
-- 
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity tmds_decoder is
    Port ( 
        tmds_data_in     : in    STD_LOGIC_VECTOR(9 downto 0);
        RGB_data_out     : out   STD_LOGIC_VECTOR(7 downto 0)
    );
end tmds_decoder;

architecture Behavioral of tmds_decoder is

    signal stage1       : STD_LOGIC_VECTOR(7 downto 0);
    signal stage2_xor   : STD_LOGIC_VECTOR(7 downto 0);
    signal stage2_xnor  : STD_LOGIC_VECTOR(7 downto 0);
    signal data_out     : STD_LOGIC_VECTOR(7 downto 0);
    
begin

    RGB_data_out    <= data_out;
    
--##--##--##--##--## Stage1 ##--##--##--##--##--   
    stage1  <= not tmds_data_in(7 downto 0) when tmds_data_in(9) = '1' else 
                   tmds_data_in(7 downto 0);
                   
                   
--##--##--##--##--## Stage2 ##--##--##--##--##--           

--**-- if tmds_in(8) = '1' --**--  
    stage2_xor(0) <= stage1(0);
s2_xor: for I in 1 to 7 generate
    begin
        stage2_xor(I) <= stage1(I) xor stage1(I-1);
    end generate;
    
--**-- if tmds_in(8) = '0' --**--  
    stage2_xnor(0) <= stage1(0);
s2_xnor: for I in 1 to 7 generate
    begin
        stage2_xnor(I) <= stage1(I) xnor stage1(I-1);
    end generate;
    
    
--##--##--##--##--## Stage out ##--##--##--##--##--       
    data_out <= stage2_xor when tmds_data_in(8) = '1' else 
                stage2_xnor;
                
end Behavioral;
