----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 17.03.2017 15:29:25
-- Design Name: 
-- Module Name: top_mod - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_mod is
    Generic (
        GLOBAL_DEBUG_OFF    : STD_LOGIC := '0'
    );
    Port ( 
        sysclk          : in    STD_LOGIC;
        sw              : in    STD_LOGIC_VECTOR(7 downto 0);
        led             : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        
        -- HDMI_OUT
        hdmi_tx_clk_p   : out   STD_LOGIC := '0';
        hdmi_tx_clk_n   : out   STD_LOGIC := '0';
        hdmi_tx_p       : out   STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
        hdmi_tx_n       : out   STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
        hdmi_tx_hpd     : in    STD_LOGIC := '0';
        hdmi_tx_rscl    : out   STD_LOGIC := '0';
        hdmi_tx_rsda    : inout STD_LOGIC := '0';
        hdmi_tx_cec     : inout STD_LOGIC := '0';
        
        -- HDMI_IN
        hdmi_rx_clk_p   : in    STD_LOGIC := '0';
        hdmi_rx_clk_n   : in    STD_LOGIC := '0';
        hdmi_rx_txen    : out   STD_LOGIC := '1';
        hdmi_rx_p       : in    STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
        hdmi_rx_n       : in    STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
        hdmi_rx_hpa     : out   STD_LOGIC := '1';
        hdmi_rx_scl     : in    STD_LOGIC := '0';
        hdmi_rx_sda     : inout STD_LOGIC := '0';
        hdmi_rx_cec     : inout STD_LOGIC := '0'
    );
end top_mod;

architecture Behavioral of top_mod is


--##--##--##--##--## VGA to DVI --##--##--##--##--##--
    
    component VGA_to_DVI is
        Port ( 
            clk_in          : in    STD_LOGIC;
            reset_in        : in    STD_LOGIC;
            
            VGA_red_in      : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_green_in    : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_blue_in     : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_hsync_in    : in    STD_LOGIC;
            VGA_vsync_in    : in    STD_LOGIC;
            VGA_blank_in    : in    STD_LOGIC;
            
            DVI_clk_out     : out   STD_LOGIC;
            DVI_ch_out      : out   STD_LOGIC_VECTOR(2 downto 0)
        );
    end component VGA_to_DVI;
    
--**-- Output Signal --**--
    
    signal VGA_to_DVI_w_DVI_clk     : STD_LOGIC;
    signal VGA_to_DVI_w_DVI_ch      : STD_LOGIC_VECTOR(2 downto 0);


--##--##--##--##--## EDID --##--##--##--##--##--    
    
    component edid_rom is
        port ( 
            clk         : in    std_logic;
            sclk_raw    : in    std_logic;
            sdat_raw    : inout std_logic;
            edid_debug  : out std_logic_vector(2 downto 0)
        );
    end component;
    
    
    
--##--##--##--##--## DVI_to_VGA --##--##--##--##--##--

    component DVI_to_VGA is
        Port ( 
            reset_in        : in    STD_LOGIC;
            
            DVI_clk_in      : in    STD_LOGIC;
            DVI_ch_in       : in    STD_LOGIC_VECTOR(2 downto 0);
            
            VGA_clk_out     : out   STD_LOGIC := '0';
            VGA_red         : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_green       : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_blue        : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_hsync       : out   STD_LOGIC := '0';
            VGA_vsync       : out   STD_LOGIC := '0';
            VGA_blank       : out   STD_LOGIC := '0'
        );
    end component DVI_to_VGA;
    
--**-- Output Signal --**--
        
    signal DVI_to_VGA_w_VGA_clk_out     : STD_LOGIC;
    signal DVI_to_VGA_VGA_red           : STD_LOGIC_VECTOR(7 downto 0);
    signal DVI_to_VGA_VGA_green         : STD_LOGIC_VECTOR(7 downto 0);
    signal DVI_to_VGA_VGA_blue          : STD_LOGIC_VECTOR(7 downto 0);
    signal DVI_to_VGA_VGA_hsync         : STD_LOGIC;
    signal DVI_to_VGA_VGA_vsync         : STD_LOGIC;
    signal DVI_to_VGA_VGA_blank         : STD_LOGIC;


--##--##--##--##--## VGA Signal modificator --##--##--##--##--##--

    component vga_neg is
        Port (
            BGR_neg         : in    STD_LOGIC_VECTOR(2 downto 0);
        
            VGA_clk_pix_in  : in    STD_LOGIC;
            VGA_red_in      : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_green_in    : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_blue_in     : in    STD_LOGIC_VECTOR(7 downto 0);
            VGA_hsync_in    : in    STD_LOGIC;
            VGA_vsync_in    : in    STD_LOGIC;
            VGA_blank_in    : in    STD_LOGIC;
            
            VGA_clk_pix_out : out   STD_LOGIC := '0';
            VGA_red_out     : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_green_out   : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_blue_out    : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_hsync_out   : out   STD_LOGIC := '0';
            VGA_vsync_out   : out   STD_LOGIC := '0';
            VGA_blank_out   : out   STD_LOGIC := '0'
        );
    end component vga_neg;

--**-- Output Signal --**--
        
    signal vga_neg_w_VGA_clk_out       : STD_LOGIC;
    signal vga_neg_w_VGA_red           : STD_LOGIC_VECTOR(7 downto 0);
    signal vga_neg_w_VGA_green         : STD_LOGIC_VECTOR(7 downto 0);
    signal vga_neg_w_VGA_blue          : STD_LOGIC_VECTOR(7 downto 0);
    signal vga_neg_w_VGA_hsync         : STD_LOGIC;
    signal vga_neg_w_VGA_vsync         : STD_LOGIC;
    signal vga_neg_w_VGA_blank         : STD_LOGIC;
    
    
--##--##--##--##--## GLOBAL Signal --##--##--##--##--##--
    
    signal w_reset              : STD_LOGIC := '0';
    signal w_vga_clk            : STD_LOGIC := '0';
    
    signal w_DVI_dat            : STD_LOGIC_VECTOR(2 downto 0);
    signal w_DVI_clk            : STD_LOGIC;

    signal w_DVII_DE_n          : STD_LOGIC;
    
    signal w_write              : STD_LOGIC_VECTOR(127 downto 0);

begin

--##--##--##--##--## GLOBAL Signal --##--##--##--##--##--

    
    

--**--** DVI to VGA to DVI **--**--
U_VGA_to_DVI:                                         
    VGA_to_DVI Port map(                              
        clk_in              => vga_neg_w_VGA_clk_out,             
        reset_in            => w_reset,               
                               
        VGA_red_in          => vga_neg_w_VGA_red,                          
        VGA_green_in        => vga_neg_w_VGA_green,           
        VGA_blue_in         => vga_neg_w_VGA_blue,            
        VGA_hsync_in        => vga_neg_w_VGA_hsync,           
        VGA_vsync_in        => vga_neg_w_VGA_vsync,           
        VGA_blank_in        => vga_neg_w_VGA_blank,             
                                                        
        DVI_clk_out         => VGA_to_DVI_w_DVI_clk,  
        DVI_ch_out          => VGA_to_DVI_w_DVI_ch    
    );                                                



--**-- Differental buffer Output Signal --**--

out_tx0_buf: OBUFDS generic map ( IOSTANDARD => "TMDS_33",  SLEW => "FAST")
        port map ( O  => hdmi_tx_p(0), OB => hdmi_tx_n(0), I  => VGA_to_DVI_w_DVI_ch(0));
    
out_tx1_buf: OBUFDS generic map ( IOSTANDARD => "TMDS_33",  SLEW => "FAST")
        port map ( O  => hdmi_tx_p(1), OB => hdmi_tx_n(1), I  => VGA_to_DVI_w_DVI_ch(1));
    
out_tx2_buf: OBUFDS generic map ( IOSTANDARD => "TMDS_33",  SLEW => "FAST")
        port map ( O  => hdmi_tx_p(2), OB => hdmi_tx_n(2), I  => VGA_to_DVI_w_DVI_ch(2));
        
out_tx_clk: OBUFDS generic map ( IOSTANDARD => "TMDS_33",  SLEW => "FAST")
        port map ( O  => hdmi_tx_clk_p, OB => hdmi_tx_clk_n, I  => VGA_to_DVI_w_DVI_clk);    


--##--##--##--##--## EDID --##--##--##--##--##--   

u_EDID:
    edid_rom port map ( 
        clk         => sysclk,     
        sclk_raw    => hdmi_rx_scl,
        sdat_raw    => hdmi_rx_sda,
        edid_debug  => open            
    );

--##--##--##--##--## DVI_to_VGA --##--##--##--##--##--

u_DVI_to_VGA:
    DVI_to_VGA Port map ( 
        reset_in        => '0',
                           
        DVI_clk_in      => w_DVI_clk,
        DVI_ch_in       => w_DVI_dat,
                           
        VGA_clk_out     => DVI_to_VGA_w_VGA_clk_out,
        VGA_red         => DVI_to_VGA_VGA_red,
        VGA_green       => DVI_to_VGA_VGA_green,
        VGA_blue        => DVI_to_VGA_VGA_blue,
        VGA_hsync       => DVI_to_VGA_VGA_hsync,
        VGA_vsync       => DVI_to_VGA_VGA_vsync,
        VGA_blank       => DVI_to_VGA_VGA_blank    
    );


--**-- Differental buffer input Signal --**--

in_rx1_buf: IBUFDS generic map ( DIFF_TERM => FALSE, IBUF_LOW_PWR => FALSE, IOSTANDARD => "TMDS_33")
    port map ( I  => hdmi_rx_p(0), IB => hdmi_rx_n(0), O => w_DVI_dat(0));
        
in_rx2_buf: IBUFDS generic map ( DIFF_TERM => FALSE, IBUF_LOW_PWR => FALSE, IOSTANDARD => "TMDS_33")
    port map ( I  => hdmi_rx_p(1), IB => hdmi_rx_n(1), O => w_DVI_dat(1));
    
in_rx3_buf: IBUFDS generic map ( DIFF_TERM => FALSE, IBUF_LOW_PWR => FALSE, IOSTANDARD => "TMDS_33")
    port map ( I  => hdmi_rx_p(2), IB => hdmi_rx_n(2), O => w_DVI_dat(2));    
    
in_clk_buf: IBUFDS generic map ( DIFF_TERM => FALSE, IBUF_LOW_PWR => FALSE, IOSTANDARD => "TMDS_33")
        port map ( I  => hdmi_rx_clk_p, IB => hdmi_rx_clk_n, O => w_DVI_clk);   



--##--##--##--##--## VGA Signal modificator --##--##--##--##--##--

u_vga_neg:
    vga_neg Port map ( 
        
        BGR_neg             => sw(2 downto 0),
        
        VGA_clk_pix_in      => DVI_to_VGA_w_VGA_clk_out,               
        VGA_red_in          => DVI_to_VGA_VGA_red,      
        VGA_green_in        => DVI_to_VGA_VGA_green,    
        VGA_blue_in         => DVI_to_VGA_VGA_blue,     
        VGA_hsync_in        => DVI_to_VGA_VGA_hsync,    
        VGA_vsync_in        => DVI_to_VGA_VGA_vsync,                   
        VGA_blank_in        => DVI_to_VGA_VGA_blank,                    
                              
        VGA_clk_pix_out     => vga_neg_w_VGA_clk_out,
        VGA_red_out         => vga_neg_w_VGA_red,    
        VGA_green_out       => vga_neg_w_VGA_green,  
        VGA_blue_out        => vga_neg_w_VGA_blue,   
        VGA_hsync_out       => vga_neg_w_VGA_hsync, 
        VGA_vsync_out       => vga_neg_w_VGA_vsync, 
        VGA_blank_out       => vga_neg_w_VGA_blank
    );

end Behavioral;
